#ifndef IMPLICANTEPRIMO_H
#define IMPLICANTEPRIMO_H

#include <iostream>
#include <iomanip>

#include <vector>
#include <map>
#include "implicante.h"

// Clase para encapsular implicantes primos
class implicanteprimo {
public:
    // Constructor
    implicanteprimo(implicante *i, std::vector<minterm> &m);
    // Constructor copia, necesario para aplicar el algoritmo sort
    implicanteprimo(const implicanteprimo &t);
    // Operador de asignacin, necesario para aplicar el algoritmo sort
    implicanteprimo &operator=(const implicanteprimo &t);
    // Funcin esttica de comparacin usada para ordenar vectores de implicantes primos
    static bool Comparar(implicanteprimo &a, implicanteprimo &b);
    // Incrementa los elementos del mapa 'cuenta' con los minterms de este implicante implicanteprimo
    void AjustaCuenta(std::map<int,int> &cuenta);
    // Elimina los minterms de este implicante presentes en el parmetro t
    void EliminarMinterms(implicanteprimo &t);
    // Devuelve true si el implicante contiene el minterm del parmetro
    bool Contiene(minterm &m) const;
    // Interfaz
    bool Esencial() { return esencial; }
    void MarcaEsencial() { esencial = true; }
    int NMinterms() const { return nMinterms; }
    int NOperaciones() const { return nOperaciones; }
    std::map<int,bool> &Presente() { return presente; }
    implicante *Imp() { return imp; }
    // Visualizaciones, usadas en depuracin
/*    void MostrarEsencial() {
        if(esencial) {
            for(int i=imp->Tam()-1; i>=0; i--) {
                if(imp->Bit()[i] == '1') std::cout << "I" << imp->Tam()-i << " ";
                if(imp->Bit()[i] == '0') std::cout << "I" << imp->Tam()-i << "' ";
            }
        }
    }
    void Mostrar() {
        std::cout << imp->Bit();
        std::cout << "(" << std::setw(2) << nMinterms << "-" << std::setw(2) << nOperaciones << ") ";
        std::cout << "=> ";
        for (auto it = presente.begin(); it != presente.end(); ++it) {
            if(it->second) std::cout << " X "; else std::cout << "   ";
        }
        if(esencial) std::cout << " E";
    }*/
private:
    implicante *imp; // Puntero al implicante dentro del vector de implicantes
    std::map<int,bool> presente; // Indica si el implicante primo contiene o no los minterms necesarios
    bool esencial; // Implicante primo marcado como esencial
    int nMinterms; // Usado para ordenar, nmero de minterms que contiene el implicante
    int nOperaciones; // Usado para ordenar, nmero de operaciones del implicante primo
};

#endif // header guard

