#include <iostream>
#include <iomanip>

#include <algorithm>
#include <vector>
#include <map>

#include "minterm.h"
#include "implicante.h"
#include "implicanteprimo.h"

int main()
{
    int noBits = 4;
    std::vector<minterm> m = {0,1,2};
    std::vector<minterm> r = {13,14,15};
    std::vector<std::vector<implicante>> grupo;

    implicante::SetTam(noBits);

    grupo.resize(noBits+1); // Crear grupos

    for(size_t i=0; i<m.size(); i++) {
        grupo[m[i].BitsActivos()].push_back(implicante(m[i]));
    };
    for(size_t i=0; i<r.size(); i++) {
        grupo[r[i].BitsActivos()].push_back(implicante(r[i]));
    };


    // Mostrar grupos:
    for(int i=0; i<=noBits; i++) {
        std::cout << i << " bit" << std::endl;
        for(size_t j=0; j<grupo[i].size(); j++) {
            for(minterm n : grupo[i][j].Minterm() ) std::cout << n.Valor() << ",";
            std::cout << grupo[i][j].Bit();
            std::cout << std::endl;
        }
    }



    // Combinar grupos
    implicante imp;
    bool contiene;
    bool repetir;
    do {
        repetir = false;
        int nbit=0;
        // Combinar grupo de nbit con grupo de nbit+1
        while(nbit < noBits) {
            std::cout << "Combina " << nbit << " con " << nbit+1 << std::endl;
            size_t s1 = grupo[nbit].size();
            size_t s2 = grupo[nbit+1].size();
            if(s1 && s2) {
                for(size_t i=0; i<s1; i++) {
                    for(size_t j=0; j<s2; j++) {
                        if(!grupo[nbit][i].Marca() && !grupo[nbit+1][j].Marca() && grupo[nbit][i]-grupo[nbit+1][j] == 1) {
                            imp = implicante(grupo[nbit][i], grupo[nbit+1][j]);
                            contiene=false;
                            for(size_t k=0; k<grupo[nbit].size() && !contiene; k++) contiene |= grupo[nbit][k] == imp;
                            std::cout << "Minterms: ";
                            for(minterm n : grupo[nbit][i].Minterm() ) std::cout << n.Valor() << ",";
                            for(minterm n : grupo[nbit+1][j].Minterm() ) std::cout << n.Valor() << ",";
                            std::cout << " : ";
                            std::cout << grupo[nbit][i].Bit();
                            std::cout << " + ";
                            std::cout << grupo[nbit+1][j].Bit();
                            std::cout << " => ";
                            std::cout << imp.Bit();
                            std::cout << std::endl;
                            if(!contiene) {
                                grupo[nbit].push_back(imp);
                                repetir = true;
                            } else {
                                std::cout << "repetido" << std::endl;
                            }
                        }
                        if(s2 && grupo[nbit+1][j].Usado()) grupo[nbit+1][j].Marcar();
                    }
                    if(s1 && grupo[nbit][i].Usado()) grupo[nbit][i].Marcar();
                }
            }
            nbit++;
        }
        // Marcar los implicantes usados
        for(nbit=0; nbit<=noBits; nbit++) {
            size_t s1 = grupo[nbit].size();
            for(size_t i=0; i<s1; i++)
                if(grupo[nbit][i].Usado()) grupo[nbit][i].Marcar();
        }

    } while(repetir);

    std::cout << "Determinantes primos: " << std::endl;
    for(int i=0; i<=noBits; i++) {
        for(size_t j=0; j<grupo[i].size(); j++) {
            if(!grupo[i][j].Marca()) {
                for(minterm n : grupo[i][j].Minterm() ) std::cout << n.Valor() << ",";
                std::cout << grupo[i][j].Bit();
                //if(!grupo[i][j].Marca()) std::cout << "*";
                std::cout << std::endl;
            }
        }
    }

    std::vector<implicanteprimo> tablaFinal;
    std::map<int,int> cuenta; // Cuenta de presencia de minterms en determinantes primos.
    bool terminado=false;
    size_t i, j;

    // Construir tabla final:
    for(int i=0; i<=noBits; i++) {
        size_t t=grupo[i].size();
        for(size_t j=0; j<t; j++) {
            if(!grupo[i][j].Marca()) {
                tablaFinal.push_back(implicanteprimo(&grupo[i][j], m));
            }
        }
    }


    // Seleccionar determinantes esenciales:
    // Los que contengan en exclusividad uno de los minterms.
    do {
        // Tabla de determinantes primos:
        std::cout << "Tabla" << std::endl;
        for(int i=0; i<noBits; i++) std::cout << ' ';
        std::cout << "                  "; // =>
        for(size_t n=0; n<m.size(); n++) std::cout << std::setw(3) << m[n].Valor();
        std::cout << std::endl;
        for(size_t i=0; i<tablaFinal.size(); i++) {
            std::cout << tablaFinal[i].Imp()->Bit();
            std::cout << "(MI: " << std::setw(2) << tablaFinal[i].NMinterms() << "- OP:" << std::setw(2) << tablaFinal[i].NOperaciones() << ") ";
            std::cout << "=> ";
            for (auto it = tablaFinal[i].Presente().begin(); it != tablaFinal[i].Presente().end(); ++it) {
                if(it->second) std::cout << " X "; else std::cout << "   ";
            }
            if(tablaFinal[i].Esencial()) std::cout << " E";
            std::cout << std::endl;
        }
        do {
            repetir = false;
            for(size_t x=0; x<m.size(); x++) cuenta[m[x].Valor()] = 0;
            for(size_t i=0; i<tablaFinal.size(); i++) {
                if(!tablaFinal[i].Esencial()) tablaFinal[i].AjustaCuenta(cuenta);
            }
            std::cout << "------" << std::endl;
            for (auto it = cuenta.begin(); it != cuenta.end(); ++it) {
                std::cout << it->first << " :> " << it->second << std::endl;
            }
            // Buscar los determinantes con minters con cuenta igual a 1 (Si los hay)
            for(size_t x=0; x<m.size(); x++)
                if(cuenta[m[x].Valor()] == 1) { // Guarda bits:
                    repetir = true;
                    i=0;
                    while(!tablaFinal[i].Contiene(m[x]) && i<tablaFinal.size()) i++;
                    tablaFinal[i].MarcaEsencial();
                    for(j=0; j<tablaFinal.size(); j++) {
                        if(j != i) tablaFinal[j].EliminarMinterms(tablaFinal[i]);
                    }
                    tablaFinal[i].EliminarMinterms(tablaFinal[i]);
                }
        } while(repetir);
        terminado = true;
        for(j=0; terminado && j<tablaFinal.size(); j++) {
            if(!tablaFinal[j].Esencial() && tablaFinal[j].NMinterms()) terminado = false;
        }
        if(!terminado) {
            // Seleccionar determinantes prioritariamente por nmero de minterms, de mayor a menor
            // y por nmero de bits de menor a mayor (los ceros cuentas por 2)
            std::sort(tablaFinal.begin(), tablaFinal.end(), implicanteprimo::Comparar);

            // Mostrar tabla de determinantes primos:
            std::cout << "Tabla" << std::endl;
            for(int i=0; i<noBits; i++) std::cout << ' ';
            std::cout << "          "; // =>
            for(size_t n=0; n<m.size(); n++) std::cout << std::setw(3) << m[n].Valor();
            std::cout << std::endl;
            for(size_t i=0; i<tablaFinal.size(); i++) {
                std::cout << tablaFinal[i].Imp()->Bit();
                std::cout << "(" << std::setw(2) << tablaFinal[i].NMinterms() << "-" << std::setw(2) << tablaFinal[i].NOperaciones() << ") ";
                std::cout << "=> ";
                for (auto it = tablaFinal[i].Presente().begin(); it != tablaFinal[i].Presente().end(); ++it) {
                    if(it->second) std::cout << " X "; else std::cout << "   ";
                }
                if(tablaFinal[i].Esencial()) std::cout << " E";
                std::cout << std::endl;
            }

            // Seleccionamos el primero como esencial:
            for(j=1; j<tablaFinal.size(); j++) {
                tablaFinal[j].EliminarMinterms(tablaFinal[0]);
            }
            tablaFinal[0].EliminarMinterms(tablaFinal[0]);
            tablaFinal[0].MarcaEsencial();
            // Optimizacin opcional:
//            terminado = true;
//            for(j=0; terminado && j<tablaFinal.size(); j++) {
//                if(!tablaFinal[j].Esencial() && tablaFinal[j].NMinterms()) terminado = false;
//            }
        }
    } while(!terminado);

    std::cout << "Resultado" << std::endl;
    bool primero=true;
    std::cout << "S = ";
    for(size_t i=0; i<tablaFinal.size(); i++) {
        if(tablaFinal[i].Esencial()) {
            if(!primero) { std::cout << "+ "; }
            else primero=false;
            for(int j=tablaFinal[i].Imp()->Tam()-1; j>=0; j--) {
                if(tablaFinal[i].Imp()->Bit()[j] == '1') std::cout << "I" << noBits-j << " ";
                if(tablaFinal[i].Imp()->Bit()[j] == '0') std::cout << "I" << noBits-j << "' ";
            }
        }
    }
    std::cout << std::endl;
std::cin.get();
    return 0;
}
